# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0


from datetime import timedelta
from typing import TYPE_CHECKING, ClassVar, Optional

from ..model.core import Party
from .api import MetricEvents

if TYPE_CHECKING:
    from prometheus_client import Gauge
    from prometheus_client.registry import CollectorRegistry


class PrometheusMetricEvents(MetricEvents):

    _DEFAULT = None  # type: ClassVar[PrometheusMetricEvents]

    @classmethod
    def default(cls):
        if PrometheusMetricEvents._DEFAULT is not None:
            return PrometheusMetricEvents._DEFAULT
        else:
            from prometheus_client import REGISTRY

            PrometheusMetricEvents._DEFAULT = PrometheusMetricEvents(REGISTRY)
            return PrometheusMetricEvents._DEFAULT

    def __init__(self, registry: "Optional[CollectorRegistry]" = None):
        from prometheus_client import REGISTRY, Gauge

        if registry is None:
            registry = REGISTRY

        self._loop_responsiveness_seconds = Gauge(
            "dazl_network_loop_responsiveness_seconds",
            "Number of seconds to immediately schedule and execute a no-op on the event loop",
            registry=registry,
        )

    def loop_responsiveness(self, responsiveness: "timedelta") -> None:
        self._loop_responsiveness_seconds.set(responsiveness.total_seconds())

    def party_offset(self, party: "Party", offset: str) -> None:
        pass



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Core types
----------

The :mod:`dazl.model.core` module contains classes used on both the read-side and the write-side of
the Ledger API.

.. autoclass:: ContractId
   :members:
"""
from dataclasses import dataclass
from datetime import datetime
from pathlib import Path
from typing import (
    TYPE_CHECKING,
    BinaryIO,
    Callable,
    Collection,
    Dict,
    Optional,
    Tuple,
    TypeVar,
    Union,
)
import warnings

from ..damlast.daml_lf_1 import TypeConName
from ..prim import ContractData, ContractId as ContractId_, Party

if TYPE_CHECKING:
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", DeprecationWarning)
        from .types import Type, TypeReference

T = TypeVar("T")


class ContractId(ContractId_):
    __slots__ = ("_value_type_deprecated",)

    def __init__(self, contract_id: str, template_id: "Union[str, Type, TypeConName]"):
        warnings.warn(
            "dazl.model.core.ContractId is deprecated; use dazl.prim.ContractId instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        from ..damlast.compat import parse_template

        if not isinstance(contract_id, str):
            raise ValueError("contract_id must be a string")

        value = contract_id
        value_type, value_type_deprecated = parse_template(template_id)

        super().__init__(value_type, value)
        object.__setattr__(self, "_value_type_deprecated", value_type_deprecated)

    @property
    def contract_id(self) -> str:
        """
        Get the raw contract ID value (for example, ``"#4:1"``).
        """
        warnings.warn(
            "ContractId.contract_id is deprecated; use ContractId.value instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.value

    @property
    def template_id(self) -> "TypeReference":
        """
        Get the type of template that is pointed to by this :class:`ContractId` as a
        :class:`TypeReference`. Note that usage of :class:`Type` and :class:`TypeReference` are
        deprecated, and :meth:`value_type` should be used instead.

        As of dazl 7.3.0, the :class:`TemplateId` is always normalized to a :class:`TypeReference`,
        regardless of what the :class:`ContractId` was constructed with.
        """
        warnings.warn(
            "ContractId.template_id is deprecated; use ContractId.value_type instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self._value_type_deprecated

    def exercise(self, choice_name, arguments=None):
        """
        Create an :class:`ExerciseCommand` that represents the result of exercising a choice on this
        contract with the specified choice.

        :param choice_name:
            The name of the choice to exercise.
        :param arguments:
            (optional) A ``dict`` of named values to send as parameters to the choice exercise.
        """
        from .writing import ExerciseCommand

        return ExerciseCommand(self, choice_name, arguments=arguments)

    def replace(self, contract_id=None, template_id=None):
        """
        Return a new :class:`ContractId` instance replacing specified fields with values.
        """
        warnings.warn(
            "ContractId.replace is deprecated; simply construct a ContractId with the desired "
            "values instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", DeprecationWarning)
            return ContractId(
                contract_id if contract_id is not None else self.value,
                template_id if template_id is not None else self.value_type,
            )

    def for_json(self):
        """
        Return the JSON representation of this contract. This is currently just the contract ID
        string itself.
        """
        return self.value


ContractMatch = Union[None, Callable[[ContractData], bool], ContractData]
ContractsState = Dict[ContractId, ContractData]
ContractsHistoricalState = Dict[ContractId, Tuple[ContractData, bool]]


class ContractContextualDataCollection(tuple):
    def __getitem__(self, index: Union[int, str, ContractId]):
        if index is None:
            raise ValueError("the index cannot be None")
        elif isinstance(index, int):
            return tuple.__getitem__(self, index)
        elif isinstance(index, str):
            for cxd in self:
                if cxd.cid.contract_id == index:
                    return cxd
            raise KeyError(index)
        elif isinstance(index, ContractId):
            for cxd in self:
                if cxd.cid == index:
                    return cxd
            raise KeyError(index)
        else:
            raise TypeError("cannot index into a ContractContextualDataCollection with {index!r}")


@dataclass(frozen=True)
class ContractContextualData:
    cid: ContractId
    cdata: "Optional[ContractData]"
    effective_at: datetime
    archived_at: "Optional[datetime]"
    active: bool


# Wherever the API expects a DAR, we can take a file path, `bytes`, or a byte buffer.
Dar = Union[bytes, str, Path, BinaryIO]


@dataclass(frozen=True)
class SourceLocation:
    file_name: str
    start_line: int
    end_line: int


class DazlError(Exception):
    """
    Superclass of errors raised by dazl.
    """


class DazlWarning(Warning):
    """
    Superclass of warnings raised by dazl.
    """


class DazlPartyMissingError(DazlError):
    """
    Error raised when a party or some information about a party is requested, and that party is not
    found.
    """

    def __init__(self, party: Party):
        super().__init__(f"party {party!r} does not have a defined client")
        self.party = party


class DazlImportError(ImportError, DazlError):
    """
    Import error raised when an optional dependency could not be found.
    """

    def __init__(self, missing_module, message):
        super().__init__(message)
        self.missing_module = missing_module


class UserTerminateRequest(DazlError):
    """
    Raised when the user has initiated a request to terminate the application.
    """


class ConnectionTimeoutError(DazlError):
    """
    Raised when a connection failed to be established before the connection timeout elapsed.
    """


class CommandTimeoutError(DazlError):
    """
    Raised when a corresponding event for a command was not seen in the appropriate time window.
    """


class ConfigurationError(DazlError):
    """
    Raised when a configuration error prevents a client from being started.

    .. attribute:: ConfigurationError.reasons

        A collection of reasons for a failure.
    """

    def __init__(self, reasons: "Union[str, Collection[str]]"):
        if reasons is None:
            self.reasons = []  # type: Collection[str]
        elif isinstance(reasons, str):
            self.reasons = [reasons]
        else:
            self.reasons = reasons  # type: Collection[str]


class ConnectionClosedError(DazlError):
    """
    Raised when trying to do something that requires a connection after connection pools have been
    closed.
    """


class UnknownTemplateWarning(DazlWarning):
    """
    Raised when trying to do something with a template name that is unknown.
    """


class ProcessDiedException(DazlError):
    pass



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0


from typing import Optional, Sequence

from .daml_lf_1 import *

__all__ = ["ProtobufParser"]


# noinspection PyPep8Naming,PyMethodMayBeStatic
class ProtobufParser:
    def __init__(self, current_package: "PackageRef"):
        from typing import List

        self.current_package = current_package
        self.interned_strings = []  # type: List[str]
        self.interned_dotted_names = []  # type: List[List[int]]

    # noinspection PyUnusedLocal
    def parse_Unit(self, pb) -> "Unit":
        return UNIT

    def parse_ModuleRef(self, pb) -> "Optional[ModuleRef]":
        sum_name = pb.package_ref.WhichOneof("Sum")
        module_name = self._resolve_dotted_name(pb.module_name_dname, pb.module_name_interned_dname)
        if sum_name is None:
            return None
        elif sum_name == "self":
            return ModuleRef(self.current_package, module_name)
        elif sum_name == "package_id_str":
            return ModuleRef(pb.package_ref.package_id_str, module_name)
        elif sum_name == "package_id_interned_str":
            return ModuleRef(
                self.interned_strings[pb.package_ref.package_id_interned_str], module_name
            )
        else:
            raise ValueError(f"unknown sum type value: {sum_name!r}")

    def parse_TypeConName(self, pb) -> "TypeConName":
        return TypeConName(
            self.parse_ModuleRef(pb.module),
            self._resolve_dotted_name(pb.name_dname, pb.name_interned_dname).segments,
        )

    def parse_TypeSynName(self, pb) -> "TypeSynName":
        return TypeSynName(
            self.parse_ModuleRef(pb.module),
            self._resolve_dotted_name(pb.name_dname, pb.name_interned_dname).segments,
        )

    def parse_DottedName(self, pb) -> "DottedName":
        return DottedName(segments=self._resolve_string_seq(pb.segments, pb.segments_interned_id))

    def parse_ValName(self, pb) -> "ValName":
        return ValName(
            self.parse_ModuleRef(pb.module),
            self._resolve_string_seq(pb.name_dname, pb.name_interned_dname),
        )

    def parse_FieldWithType(self, pb) -> "FieldWithType":
        """A field definition in a record or a variant associated with a type."""
        return FieldWithType(
            self._resolve_string(pb.field_str, pb.field_interned_str), self.parse_Type(pb.type)
        )

    def parse_VarWithType(self, pb) -> "VarWithType":
        """Binder associated with a type."""
        return VarWithType(
            self._resolve_string(pb.var_str, pb.var_interned_str), self.parse_Type(pb.type)
        )

    def parse_TypeVarWithKind(self, pb) -> "TypeVarWithKind":
        return TypeVarWithKind(
            self._resolve_string(pb.var_str, pb.var_interned_str), self.parse_Kind(pb.kind)
        )

    def parse_FieldWithExpr(self, pb) -> "FieldWithExpr":
        return FieldWithExpr(
            self._resolve_string(pb.field_str, pb.field_interned_str), self.parse_Expr(pb.expr)
        )

    def parse_Binding(self, pb) -> "Binding":
        return Binding(self.parse_VarWithType(pb.binder), self.parse_Expr(pb.bound))

    def parse_Kind(self, pb) -> "Optional[Kind]":
        sum_name = pb.WhichOneof("Sum")
        if sum_name is None:
            return None
        elif sum_name == "star":
            return Kind(star=self.parse_Unit(pb.star))
        elif sum_name == "arrow":
            return Kind(arrow=self.parse_Kind_Arrow(pb.arrow))
        elif sum_name == "nat":
            return Kind(nat=pb.nat)
        else:
            raise ValueError(f"unknown sum type value: {sum_name!r}")

    def parse_Kind_Arrow(self, pb) -> "Kind.Arrow":
        return Kind.Arrow(
            tuple(self.parse_Kind(param) for param in pb.params), self.parse_Kind(pb.result)
        )

    def parse_PrimType(self, pb) -> "PrimType":
        return PrimType(pb)

    def parse_Type(self, pb) -> "Type":
        sum_name = pb.WhichOneof("Sum")
        if sum_name == "var":
            return self.parse_Type_Var(pb.var)
        elif sum_name == "con":
            return self.parse_Type_Con(pb.con)
        elif sum_name == "prim":
            return Type(prim=self.parse_Type_Prim(pb.prim))
        elif sum_name == "fun":
            return self.parse_Type_Fun(pb.fun)
        elif sum_name == "forall":
            return Type(forall=self.parse_Type_Forall(pb.forall))
        elif sum_name == "struct":
            return Type(tuple=self.parse_Type_Tuple(pb.struct))
        elif sum_name == "nat":
            return Type(nat=pb.nat)
        elif sum_name == "syn":
            return Type(syn=self.parse_Type_Syn(pb.syn))
        else:
            raise ValueError(f"unknown sum type value: {sum_name!r}")

    def parse_Type_Var(self, pb) -> "Type":
        return Type(
            var=Type.Var(
                self._resolve_string(pb.var_str, pb.var_interned_str),
                tuple(self.parse_Type(arg) for arg in pb.args),
            )
        )

    def parse_Type_Con(self, pb) -> "Type":
        """
        Create a :class:`Type` instance (but may produce something slightly different than the AST
        due to ``Map``/``Optional`` type rewriting).
        """
        tycon = self.parse_TypeConName(pb.tycon)
        args = tuple(self.parse_Type(arg) for arg in pb.args)
        core_type = Type(con=Type.Con(tycon, args))
        return core_type

    def parse_Type_Prim(self, pb) -> "Type.Prim":
        return Type.Prim(
            self.parse_PrimType(pb.prim), tuple(self.parse_Type(arg) for arg in pb.args)
        )

    def parse_Type_Fun(self, pb) -> "Type":
        """``fun`` has been deprecated and this code path is now replaced with PrimType.ARROW"""
        last_type = self.parse_Type(pb.result)
        for param in reversed(pb.params):
            last_type = Type(prim=Type.Prim(PrimType.ARROW, (self.parse_Type(param), last_type)))
        return last_type

    def parse_Type_Forall(self, pb) -> "Type.Forall":
        return Type.Forall(
            tuple(self.parse_TypeVarWithKind(var) for var in pb.vars), self.parse_Type(pb.body)
        )

    def parse_Type_Tuple(self, pb) -> "Type.Tuple":
        return Type.Tuple(tuple(self.parse_FieldWithType(field) for field in pb.fields))

    def parse_Type_Syn(self, pb) -> "Type.Syn":
        return Type.Syn(
            tysyn=self.parse_TypeSynName(pb.tysyn),
            args=tuple(self.parse_Type(arg) for arg in pb.args),
        )

    def parse_PrimCon(self, pb) -> "Optional[PrimCon]":
        if pb is None:
            return None
        elif pb == 0:
            return PrimCon.CON_UNIT
        elif pb == 1:
            return PrimCon.CON_FALSE
        elif pb == 2:
            return PrimCon.CON_TRUE
        else:
            raise ValueError(f"unknown enum value: {pb!r}")

    def parse_BuiltinFunction(self, pb) -> "Optional[BuiltinFunction]":
        if pb is None:
            return None
        return BuiltinFunction(pb)

    def parse_PrimLit(self, pb) -> "PrimLit":
        sum_name = pb.WhichOneof("Sum")
        if sum_name == "int64":
            return PrimLit(int64=pb.int64)
        elif sum_name == "decimal_str":
            return PrimLit(decimal=pb.decimal_str)
        elif sum_name == "numeric_interned_str":
            return PrimLit(numeric=self.interned_strings[pb.numeric_interned_str])
        elif sum_name == "text_str":
            return PrimLit(text=pb.text_str)
        elif sum_name == "text_interned_str":
            return PrimLit(text=self.interned_strings[pb.text_interned_str])
        elif sum_name == "timestamp":
            return PrimLit(timestamp=pb.timestamp)
        elif sum_name == "party_str":
            return PrimLit(party=pb.party_str)
        elif sum_name == "party_interned_str":
            return PrimLit(party=pb.party_interned_str)
        elif sum_name == "date":
            return PrimLit(date=pb.date)
        else:
            raise ValueError(f"unknown Sum value: {pb!r}")

    def parse_Expr(self, pb, optional: bool = False) -> "Expr":
        location = self.parse_Location(pb.location) if pb.HasField("location") else None
        if location is not None:
            args = {"location": location}
        else:
            args = {}

        sum_name = pb.WhichOneof("Sum")
        if optional and sum_name is None:
            return None
        if sum_name == "var_str":
            args["var"] = pb.var_str
        elif sum_name == "var_interned_str":
            args["var"] = self.interned_strings[pb.var_interned_str]
        elif sum_name == "val":
            args["val"] = self.parse_ValName(pb.val)
        elif sum_name == "builtin":
            args["builtin"] = self.parse_BuiltinFunction(pb.builtin)
        elif sum_name == "prim_con":
            args["prim_con"] = self.parse_PrimCon(pb.prim_con)
        elif sum_name == "prim_lit":
            args["prim_lit"] = self.parse_PrimLit(pb.prim_lit)
        elif sum_name == "rec_con":
            args["rec_con"] = self.parse_Expr_RecCon(pb.rec_con)
        elif sum_name == "rec_proj":
            args["rec_proj"] = self.parse_Expr_RecProj(pb.rec_proj)
        elif sum_name == "variant_con":
            args["variant_con"] = self.parse_Expr_VariantCon(pb.variant_con)
        elif sum_name == "struct_con":
            args["struct_con"] = self.parse_Expr_TupleCon(pb.struct_con)
        elif sum_name == "enum_con":
            args["enum_con"] = self.parse_Expr_EnumCon(pb.enum_con)
        elif sum_name == "struct_proj":
            args["struct_proj"] = self.parse_Expr_TupleProj(pb.struct_proj)
        elif sum_name == "app":
            args["app"] = self.parse_Expr_App(pb.app)
        elif sum_name == "ty_app":
            args["ty_app"] = self.parse_Expr_TyApp(pb.ty_app)
        elif sum_name == "abs":
            args["abs"] = self.parse_Expr_Abs(pb.abs)
        elif sum_name == "ty_abs":
            args["ty_abs"] = self.parse_Expr_TyAbs(pb.ty_abs)
        elif sum_name == "case":
            args["case"] = self.parse_Case(pb.case)
        elif sum_name == "let":
            args["let"] = self.parse_Block(pb.let)
        elif sum_name == "nil":
            args["nil"] = self.parse_Expr_Nil(pb.nil)
        elif sum_name == "cons":
            args["cons"] = self.parse_Expr_Cons(pb.cons)
        elif sum_name == "update":
            args["update"] = self.parse_Update(pb.update)
        elif sum_name == "scenario":
            args["scenario"] = self.parse_Scenario(pb.scenario)
        elif sum_name == "rec_upd":
            args["rec_upd"] = self.parse_Expr_RecUpd(pb.rec_upd)
        elif sum_name == "struct_upd":
            args["struct_upd"] = self.parse_Expr_TupleUpd(pb.struct_upd)
        elif sum_name == "optional_none":
            args["optional_none"] = self.parse_Expr_OptionalNone(pb.optional_none)
        elif sum_name == "optional_some":
            args["optional_some"] = self.parse_Expr_OptionalSome(pb.optional_some)
        elif sum_name == "to_any":
            args["to_any"] = self.parse_Expr_ToAny(pb.to_any)
        elif sum_name == "from_any":
            args["from_any"] = self.parse_Expr_FromAny(pb.from_any)
        elif sum_name == "to_text_template_id":
            args["to_text_template_id"] = self.parse_Expr_ToTextTemplateId(pb.to_text_template_id)
        elif sum_name == "type_rep":
            args["type_rep"] = self.parse_Type(pb.type_rep)
        else:
            raise ValueError(f"Unknown type of Expr: {sum_name!r}")

        return Expr(**args)

    def parse_Expr_RecCon(self, pb) -> "Expr.RecCon":
        return Expr.RecCon(
            self.parse_Type_Con(pb.tycon).con,
            tuple(self.parse_FieldWithExpr(field) for field in pb.fields),
        )  # length > 0

    def parse_Expr_RecProj(self, pb) -> "Expr.RecProj":
        return Expr.RecProj(
            self.parse_Type_Con(pb.tycon).con,  # Always fully applied
            self._resolve_string(pb.field_str, pb.field_interned_str),
            self.parse_Expr(pb.record),
        )

    def parse_Expr_RecUpd(self, pb) -> "Expr.RecUpd":
        """Set ``field`` in ``record`` to ``update``."""
        return Expr.RecUpd(
            self.parse_Type_Con(pb.tycon).con,
            self._resolve_string(pb.field_str, pb.field_interned_str),
            self.parse_Expr(pb.record),
            self.parse_Expr(pb.update),
        )

    def parse_Expr_VariantCon(self, pb) -> "Expr.VariantCon":
        return Expr.VariantCon(
            self.parse_Type_Con(pb.tycon).con,  # Always fully applied
            self._resolve_string(pb.variant_con_str, pb.variant_con_interned_str),
            self.parse_Expr(pb.variant_arg),
        )

    def parse_Expr_TupleCon(self, pb) -> "Expr.StructCon":
        return Expr.StructCon(
            tuple(self.parse_FieldWithExpr(field) for field in pb.fields)
        )  # length > 0

    def parse_Expr_EnumCon(self, pb) -> "Expr.EnumCon":
        return Expr.EnumCon(
            self.parse_TypeConName(pb.tycon),
            self._resolve_string(pb.enum_con_str, pb.enum_con_interned_str),
        )

    def parse_Expr_TupleProj(self, pb) -> "Expr.StructProj":
        return Expr.StructProj(
            self._resolve_string(pb.field_str, pb.field_interned_str), self.parse_Expr(pb.struct)
        )

    def parse_Expr_TupleUpd(self, pb) -> "Expr.StructUpd":
        """Set ``field`` in ``tuple`` to ``update``."""
        return Expr.StructUpd(
            self._resolve_string(pb.field_str, pb.field_interned_str),
            self.parse_Expr(pb.struct),
            self.parse_Expr(pb.update),
        )

    def parse_Expr_App(self, pb) -> "Expr.App":
        return Expr.App(
            self.parse_Expr(pb.fun), tuple(self.parse_Expr(arg) for arg in pb.args)
        )  # length > 0

    def parse_Expr_TyApp(self, pb) -> "Expr.TyApp":
        return Expr.TyApp(
            self.parse_Expr(pb.expr), tuple(self.parse_Type(type) for type in pb.types)
        )  # length > 0

    def parse_Expr_Abs(self, pb):
        return Expr.Abs(
            tuple(self.parse_VarWithType(param) for param in pb.param),  # length > 0
            self.parse_Expr(pb.body),
        )

    def parse_Expr_TyAbs(self, pb):
        return Expr.TyAbs(
            tuple(self.parse_TypeVarWithKind(param) for param in pb.param),  # length > 0
            self.parse_Expr(pb.body),
        )

    def parse_Expr_Nil(self, pb):
        return Expr.Nil(self.parse_Type(pb.type))

    def parse_Expr_Cons(self, pb):
        return Expr.Cons(
            self.parse_Type(pb.type),
            tuple(self.parse_Expr(front) for front in pb.front),  # length > 0
            self.parse_Expr(pb.tail),
        )

    def parse_Expr_OptionalNone(self, pb):
        return Expr.OptionalNone(self.parse_Type(pb.type))

    def parse_Expr_OptionalSome(self, pb):
        return Expr.OptionalSome(self.parse_Type(pb.type), self.parse_Expr(pb.body))

    def parse_Expr_ToAny(self, pb):
        return Expr.ToAny(self.parse_Type(pb.type), self.parse_Expr(pb.expr))

    def parse_Expr_FromAny(self, pb):
        return Expr.FromAny(self.parse_Type(pb.type), self.parse_Expr(pb.expr))

    def parse_Expr_ToTextTemplateId(self, pb):
        return Expr.ToTextTemplateId(self.parse_Type(pb.type))

    def parse_CaseAlt(self, pb) -> "CaseAlt":
        body = self.parse_Expr(pb.body)
        sum_name = pb.WhichOneof("Sum")
        if sum_name == "default":
            return CaseAlt(default=self.parse_Unit(pb.default), body=body)
        elif sum_name == "variant":
            return CaseAlt(variant=self.parse_CaseAlt_Variant(pb.variant), body=body)
        elif sum_name == "prim_con":
            return CaseAlt(prim_con=self.parse_PrimCon(pb.prim_con), body=body)
        elif sum_name == "nil":
            return CaseAlt(nil=self.parse_Unit(pb.nil), body=body)
        elif sum_name == "cons":
            return CaseAlt(cons=self.parse_CaseAlt_Cons(pb.cons), body=body)
        elif sum_name == "optional_none":
            return CaseAlt(optional_none=self.parse_Unit(pb.optional_none), body=body)
        elif sum_name == "optional_some":
            return CaseAlt(
                optional_some=self.parse_CaseAlt_OptionalSome(pb.optional_some), body=body
            )
        elif sum_name == "enum":
            return CaseAlt(enum=self.parse_CaseAlt_Enum(pb.enum), body=body)
        else:
            raise ValueError(f"unknown Sum value: {sum_name!r}")

    def parse_CaseAlt_Variant(self, pb) -> "CaseAlt.Variant":
        return CaseAlt.Variant(
            self.parse_TypeConName(pb.con),
            self._resolve_string(pb.variant_str, pb.variant_interned_str),
            self._resolve_string(pb.binder_str, pb.binder_interned_str),
        )

    def parse_CaseAlt_Enum(self, pb) -> "CaseAlt.Enum":
        return CaseAlt.Enum(
            self.parse_TypeConName(pb.con),
            self._resolve_string(pb.constructor_str, pb.constructor_interned_str),
        )

    def parse_CaseAlt_Cons(self, pb) -> "CaseAlt.Cons":
        return CaseAlt.Cons(
            self._resolve_string(pb.var_head_str, pb.var_head_interned_str),
            self._resolve_string(pb.var_tail_str, pb.var_tail_interned_str),
        )

    def parse_CaseAlt_OptionalSome(self, pb) -> "CaseAlt.OptionalSome":
        return CaseAlt.OptionalSome(self._resolve_string(pb.var_body_str, pb.var_body_interned_str))

    def parse_Case(self, pb) -> "Case":
        return Case(self.parse_Expr(pb.scrut), tuple(self.parse_CaseAlt(alt) for alt in pb.alts))

    def parse_Block(self, pb) -> "Block":
        return Block(
            tuple(self.parse_Binding(binding) for binding in pb.bindings), self.parse_Expr(pb.body)
        )

    def parse_Pure(self, pb) -> "Pure":
        return Pure(type=self.parse_Type(pb.type), expr=self.parse_Expr(pb.expr))

    def parse_Update(self, pb) -> "Update":
        sum_name = pb.WhichOneof("Sum")
        if sum_name == "pure":
            return Update(pure=self.parse_Pure(pb.pure))
        elif sum_name == "block":
            return Update(block=self.parse_Block(pb.block))
        elif sum_name == "create":
            return Update(create=self.parse_Update_Create(pb.create))
        elif sum_name == "exercise":
            return Update(exercise=self.parse_Update_Exercise(pb.exercise))
        elif sum_name == "fetch":
            return Update(fetch=self.parse_Update_Fetch(pb.fetch))
        elif sum_name == "get_time":
            return Update(get_time=self.parse_Unit(pb.get_time))
        elif sum_name == "embed_expr":
            return Update(embed_expr=self.parse_Update_EmbedExpr(pb.embed_expr))
        elif sum_name == "lookup_by_key":
            return Update(lookup_by_key=self.parse_Update_RetrieveByKey(pb.lookup_by_key))
        elif sum_name == "fetch_by_key":
            return Update(fetch_by_key=self.parse_Update_RetrieveByKey(pb.fetch_by_key))
        else:
            raise ValueError(f"unknown Sum value: {sum_name!r}")

    def parse_Update_Create(self, pb) -> "Update.Create":
        return Update.Create(
            template=self.parse_TypeConName(pb.template), expr=self.parse_Expr(pb.expr)
        )

    def parse_Update_Exercise(self, pb) -> "Update.Exercise":
        return Update.Exercise(
            template=self.parse_TypeConName(pb.template),
            choice=self._resolve_string(pb.choice_str, pb.choice_interned_str),
            cid=self.parse_Expr(pb.cid),
            arg=self.parse_Expr(pb.arg),
        )

    def parse_Update_Fetch(self, pb) -> "Update.Fetch":
        return Update.Fetch(
            template=self.parse_TypeConName(pb.template), cid=self.parse_Expr(pb.cid)
        )

    def parse_Update_EmbedExpr(self, pb) -> "Update.EmbedExpr":
        return Update.EmbedExpr(type=self.parse_Type(pb.type), body=self.parse_Expr(pb.body))

    def parse_Update_RetrieveByKey(self, pb) -> "Update.RetrieveByKey":
        return Update.RetrieveByKey(
            template=self.parse_TypeConName(pb.template), key=self.parse_Expr(pb.key)
        )

    def parse_Scenario(self, pb) -> "Scenario":
        sum_name = pb.WhichOneof("Sum")
        if sum_name == "pure":
            return Scenario(pure=self.parse_Pure(pb.pure))
        elif sum_name == "block":
            return Scenario(block=self.parse_Block(pb.block))
        elif sum_name == "commit":
            return Scenario(commit=self.parse_Scenario_Commit(pb.commit))
        elif sum_name == "mustFailAt":
            return Scenario(must_fail_at=self.parse_Scenario_Commit(pb.mustFailAt))
        elif sum_name == "pass":
            return Scenario(must_fail_at=self.parse_Expr(getattr(pb, "pass")))
        elif sum_name == "get_time":
            return Scenario(get_time=self.parse_Unit(pb.get_time))
        elif sum_name == "get_party":
            return Scenario(get_party=self.parse_Expr(pb.get_party))
        elif sum_name == "embed_expr":
            return Scenario(embed_expr=self.parse_Scenario_EmbedExpr(pb.embed_expr))
        else:
            raise ValueError("unknown Sum value")

    def parse_Scenario_Commit(self, pb) -> "Scenario.Commit":
        return Scenario.Commit(
            party=self.parse_Expr(pb.party),
            expr=self.parse_Expr(pb.expr),
            ret_type=self.parse_Type(pb.ret_type),
        )

    def parse_Scenario_EmbedExpr(self, pb) -> "Scenario.EmbedExpr":
        return Scenario.EmbedExpr(type=self.parse_Type(pb.type), body=self.parse_Expr(pb.body))

    def parse_Location(self, pb) -> "Location":
        return Location(self.parse_ModuleRef(pb.module), self.parse_Location_Range(pb.range))

    def parse_Location_Range(self, pb) -> "Location.Range":
        return Location.Range(pb.start_line, pb.start_col, pb.end_line, pb.end_col)

    def parse_TemplateChoice(self, pb) -> "TemplateChoice":
        return TemplateChoice(
            name=self._resolve_string(pb.name_str, pb.name_interned_str),
            consuming=pb.consuming,
            controllers=self.parse_Expr(pb.controllers),
            arg_binder=self.parse_VarWithType(pb.arg_binder),
            ret_type=self.parse_Type(pb.ret_type),
            update=self.parse_Expr(pb.update),
            self_binder=self._resolve_string(pb.self_binder_str, pb.self_binder_interned_str),
            location=self.parse_Location(pb.location),
        )

    def parse_KeyExpr(self, pb) -> "KeyExpr":
        if pb.HasField("projections"):
            return KeyExpr(projections=self.parse_KeyExpr_Projections(pb.projections))
        elif pb.HasField("record"):
            return KeyExpr(record=self.parse_KeyExpr_Record(pb.record))
        else:
            raise ValueError(f"unknown KeyExpr {pb}")

    def parse_KeyExpr_Projection(self, pb) -> "KeyExpr.Projection":
        return KeyExpr.Projection(
            tycon=self.parse_Type_Con(pb.tycon).con,
            field=self._resolve_string(pb.name, pb.interned_id),
        )

    def parse_KeyExpr_Projections(self, pb) -> "KeyExpr.Projections":
        return KeyExpr.Projections(
            projections=[self.parse_KeyExpr_Projection(p) for p in pb.projections]
        )

    def parse_KeyExpr_RecordField(self, pb) -> "KeyExpr.RecordField":
        return KeyExpr.RecordField(
            field=self._resolve_string(pb.name, pb.interned_id), expr=pb.expr
        )

    def parse_KeyExpr_Record(self, pb) -> "KeyExpr.Record":
        return KeyExpr.Record(
            tycon=self.parse_Type_Con(pb.tycon).con,
            fields=[self.parse_KeyExpr_RecordField(p) for p in pb.fields],
        )

    def parse_DefTemplate(self, pb) -> "DefTemplate":
        return DefTemplate(
            tycon=self._resolve_dotted_name(pb.tycon_dname, pb.tycon_interned_dname),
            param=self._resolve_string(pb.param_str, pb.param_interned_str),
            precond=self.parse_Expr(pb.precond),
            signatories=self.parse_Expr(pb.signatories),
            agreement=self.parse_Expr(pb.agreement),
            choices=tuple(self.parse_TemplateChoice(choice) for choice in pb.choices),
            observers=self.parse_Expr(pb.observers),
            location=self.parse_Location(pb.location),
            key=self.parse_DefTemplate_DefKey(pb.key) if pb.HasField("key") else None,
        )

    def parse_DefTemplate_DefKey(self, pb) -> "DefTemplate.DefKey":
        kwargs = dict(type=self.parse_Type(pb.type), maintainers=self.parse_Expr(pb.maintainers))
        key_expr_name = pb.WhichOneof("key_expr")
        if key_expr_name == "key":
            kwargs["key"] = self.parse_KeyExpr(pb.key)
        elif key_expr_name == "complex_key":
            kwargs["complex_key"] = self.parse_Expr(pb.complex_key)
        return DefTemplate.DefKey(**kwargs)

    def parse_DefDataType(self, pb) -> "DefDataType":
        kwargs = dict(
            name=self._resolve_dotted_name(pb.name_dname, pb.name_interned_dname),
            params=tuple(self.parse_TypeVarWithKind(param) for param in pb.params),
            serializable=pb.serializable,
            location=self.parse_Location(pb.location),
        )
        DataCons_name = pb.WhichOneof("DataCons")
        if DataCons_name == "record":
            kwargs["record"] = self.parse_DefDataType_Fields(pb.record)
        elif DataCons_name == "variant":
            kwargs["variant"] = self.parse_DefDataType_Fields(pb.variant)
        elif DataCons_name == "enum":
            kwargs["enum"] = self.parse_DefDataType_EnumConstructors(pb.enum)
        elif DataCons_name == "synonym":
            kwargs["synonym"] = self.parse_Type(pb.synonym)
        else:
            raise ValueError(f"unknown DataCons value: {DataCons_name!r}")
        return DefDataType(**kwargs)

    def parse_DefDataType_Fields(self, pb) -> "DefDataType.Fields":
        return DefDataType.Fields(
            fields=tuple(self.parse_FieldWithType(field) for field in pb.fields)
        )

    def parse_DefDataType_EnumConstructors(self, pb) -> "DefDataType.EnumConstructors":
        ctors_1 = tuple(pb.constructors_str)
        ctors_2 = tuple(self.interned_strings[idx] for idx in pb.constructors_interned_str)
        return DefDataType.EnumConstructors(constructors=ctors_1 + ctors_2)

    def parse_DefTypeSyn(self, pb) -> "DefTypeSyn":
        return DefTypeSyn(
            name=self._resolve_dotted_name(pb.name_dname, pb.name_interned_dname),
            params=tuple(self.parse_TypeVarWithKind(param) for param in pb.params),
            type=self.parse_Type(pb.type),
            location=self.parse_Location(pb.location),
        )

    def parse_DefValue(self, pb) -> "DefValue":
        return DefValue(
            name_with_type=self.parse_DefValue_NameWithType(pb.name_with_type),
            expr=self.parse_Expr(pb.expr),
            no_party_literals=pb.no_party_literals,
            is_test=pb.is_test,
            location=self.parse_Location(pb.location),
        )

    def parse_DefValue_NameWithType(self, pb) -> "DefValue.NameWithType":
        return DefValue.NameWithType(
            name=self._resolve_string_seq(pb.name_dname, pb.name_interned_dname),
            type=self.parse_Type(pb.type),
        )

    def parse_FeatureFlags(self, pb) -> "FeatureFlags":
        return FeatureFlags(
            forbidPartyLiterals=pb.forbidPartyLiterals,
            dontDivulgeContractIdsInCreateArguments=pb.dontDivulgeContractIdsInCreateArguments,
            dontDiscloseNonConsumingChoicesToObservers=pb.dontDiscloseNonConsumingChoicesToObservers,
        )

    def parse_Module(self, pb) -> "Module":
        return Module(
            name=self._resolve_dotted_name(pb.name_dname, pb.name_interned_dname),
            flags=self.parse_FeatureFlags(pb.flags),
            synonyms=tuple(self.parse_DefTypeSyn(value) for value in pb.synonyms),
            data_types=tuple(self.parse_DefDataType(data_type) for data_type in pb.data_types),
            values=tuple(self.parse_DefValue(value) for value in pb.values),
            templates=tuple(self.parse_DefTemplate(template) for template in pb.templates),
        )

    def parse_Package(self, pb) -> "Package":
        # TODO: this modifies state in a parser which is less than ideal; a better pattern would be
        #  to create a sub-parser with the contextual state required to understand interned package
        #  IDs
        self.interned_strings.extend(pb.interned_strings)

        indices = [
            tuple(self.interned_strings[idx] for idx in idn.segments_interned_str)
            for idn in pb.interned_dotted_names
        ]

        self.interned_dotted_names.extend(indices)

        return Package(
            modules=tuple(self.parse_Module(module) for module in pb.modules),
            metadata=self.parse_PackageMetadata(pb.metadata) if pb.HasField("metadata") else None,
        )

    def parse_PackageMetadata(self, pb) -> "PackageMetadata":
        return PackageMetadata(
            name=self.interned_strings[pb.name_interned_str],
            version=self.interned_strings[pb.version_interned_str],
        )

    def _resolve_string(self, name: "Optional[str]", interned_id: "Optional[int]") -> str:
        return name if name else self.interned_strings[interned_id]

    def _resolve_string_seq(
        self, name: "Optional[Sequence[str]]", name_interned_id: "Optional[int]"
    ) -> "Sequence[str]":
        if self.interned_dotted_names:
            return tuple(name) if name else self.interned_dotted_names[name_interned_id]
        else:
            return tuple(name) if name else tuple()

    def _resolve_dotted_name(self, pb_dotted_name, interned_id: "Optional[int]") -> "DottedName":
        return DottedName(self._resolve_string_seq(pb_dotted_name.segments, interned_id))



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/trace_context.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/daml/ledger/api/v1/trace_context.proto',
  package='com.daml.ledger.api.v1',
  syntax='proto3',
  serialized_options=b'\n\026com.daml.ledger.api.v1B\026TraceContextOuterClass\252\002\026Com.Daml.Ledger.Api.V1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n*com/daml/ledger/api/v1/trace_context.proto\x12\x16\x63om.daml.ledger.api.v1\x1a\x1egoogle/protobuf/wrappers.proto\"\x9b\x01\n\x0cTraceContext\x12\x19\n\rtrace_id_high\x18\x01 \x01(\x04\x42\x02\x30\x01\x12\x14\n\x08trace_id\x18\x02 \x01(\x04\x42\x02\x30\x01\x12\x13\n\x07span_id\x18\x03 \x01(\x04\x42\x02\x30\x01\x12\x34\n\x0eparent_span_id\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.UInt64Value\x12\x0f\n\x07sampled\x18\x05 \x01(\x08\x42I\n\x16\x63om.daml.ledger.api.v1B\x16TraceContextOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])




_TRACECONTEXT = _descriptor.Descriptor(
  name='TraceContext',
  full_name='com.daml.ledger.api.v1.TraceContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trace_id_high', full_name='com.daml.ledger.api.v1.TraceContext.trace_id_high', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trace_id', full_name='com.daml.ledger.api.v1.TraceContext.trace_id', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='span_id', full_name='com.daml.ledger.api.v1.TraceContext.span_id', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'0\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parent_span_id', full_name='com.daml.ledger.api.v1.TraceContext.parent_span_id', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sampled', full_name='com.daml.ledger.api.v1.TraceContext.sampled', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=103,
  serialized_end=258,
)

_TRACECONTEXT.fields_by_name['parent_span_id'].message_type = google_dot_protobuf_dot_wrappers__pb2._UINT64VALUE
DESCRIPTOR.message_types_by_name['TraceContext'] = _TRACECONTEXT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TraceContext = _reflection.GeneratedProtocolMessageType('TraceContext', (_message.Message,), {
  'DESCRIPTOR' : _TRACECONTEXT,
  '__module__' : 'com.daml.ledger.api.v1.trace_context_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.TraceContext)
  })
_sym_db.RegisterMessage(TraceContext)


DESCRIPTOR._options = None
_TRACECONTEXT.fields_by_name['trace_id_high']._options = None
_TRACECONTEXT.fields_by_name['trace_id']._options = None
_TRACECONTEXT.fields_by_name['span_id']._options = None
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Types that describe the behavior of the ledger itself.
"""
from dataclasses import dataclass
from typing import TYPE_CHECKING
import warnings

from .writing import Serializer

if TYPE_CHECKING:
    from ..client.pkg_loader import PackageLoader

    with warnings.catch_warnings():
        warnings.simplefilter("ignore", DeprecationWarning)
        from .types_store import PackageStore


@dataclass(init=False, frozen=True)
class LedgerMetadata:
    """
    Attributes that are invariant with respect to any party on the ledger.
    """

    ledger_id: str
    package_loader: "PackageLoader"
    serializer: "Serializer"
    protocol_version: str
    _store: "PackageStore"

    def __init__(self, ledger_id, package_loader, serializer, protocol_version, store):
        object.__setattr__(self, "ledger_id", ledger_id)
        object.__setattr__(self, "package_loader", package_loader)
        object.__setattr__(self, "serializer", serializer)
        object.__setattr__(self, "protocol_version", protocol_version)
        object.__setattr__(self, "_store", store)

    @property
    def store(self) -> "PackageStore":
        if self._store is None:
            raise Exception("eager_package_fetch is disabled, which disables the PackageStore")

        warnings.warn(
            "PackageStore is deprecated; use SymbolLookup "
            "(accessible from Network.lookup) instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self._store



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/testing/time_service.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/daml/ledger/api/v1/testing/time_service.proto',
  package='com.daml.ledger.api.v1.testing',
  syntax='proto3',
  serialized_options=b'\n\036com.daml.ledger.api.v1.testingB\025TimeServiceOuterClass\252\002\036Com.Daml.Ledger.Api.V1.Testing',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n1com/daml/ledger/api/v1/testing/time_service.proto\x12\x1e\x63om.daml.ledger.api.v1.testing\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"#\n\x0eGetTimeRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\"C\n\x0fGetTimeResponse\x12\x30\n\x0c\x63urrent_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x83\x01\n\x0eSetTimeRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x30\n\x0c\x63urrent_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08new_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp2\xce\x01\n\x0bTimeService\x12l\n\x07GetTime\x12..com.daml.ledger.api.v1.testing.GetTimeRequest\x1a/.com.daml.ledger.api.v1.testing.GetTimeResponse0\x01\x12Q\n\x07SetTime\x12..com.daml.ledger.api.v1.testing.SetTimeRequest\x1a\x16.google.protobuf.EmptyBX\n\x1e\x63om.daml.ledger.api.v1.testingB\x15TimeServiceOuterClass\xaa\x02\x1e\x43om.Daml.Ledger.Api.V1.Testingb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_GETTIMEREQUEST = _descriptor.Descriptor(
  name='GetTimeRequest',
  full_name='com.daml.ledger.api.v1.testing.GetTimeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ledger_id', full_name='com.daml.ledger.api.v1.testing.GetTimeRequest.ledger_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=147,
  serialized_end=182,
)


_GETTIMERESPONSE = _descriptor.Descriptor(
  name='GetTimeResponse',
  full_name='com.daml.ledger.api.v1.testing.GetTimeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='current_time', full_name='com.daml.ledger.api.v1.testing.GetTimeResponse.current_time', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=184,
  serialized_end=251,
)


_SETTIMEREQUEST = _descriptor.Descriptor(
  name='SetTimeRequest',
  full_name='com.daml.ledger.api.v1.testing.SetTimeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ledger_id', full_name='com.daml.ledger.api.v1.testing.SetTimeRequest.ledger_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='current_time', full_name='com.daml.ledger.api.v1.testing.SetTimeRequest.current_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='new_time', full_name='com.daml.ledger.api.v1.testing.SetTimeRequest.new_time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=254,
  serialized_end=385,
)

_GETTIMERESPONSE.fields_by_name['current_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SETTIMEREQUEST.fields_by_name['current_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SETTIMEREQUEST.fields_by_name['new_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['GetTimeRequest'] = _GETTIMEREQUEST
DESCRIPTOR.message_types_by_name['GetTimeResponse'] = _GETTIMERESPONSE
DESCRIPTOR.message_types_by_name['SetTimeRequest'] = _SETTIMEREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetTimeRequest = _reflection.GeneratedProtocolMessageType('GetTimeRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMEREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.testing.time_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.testing.GetTimeRequest)
  })
_sym_db.RegisterMessage(GetTimeRequest)

GetTimeResponse = _reflection.GeneratedProtocolMessageType('GetTimeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMERESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.testing.time_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.testing.GetTimeResponse)
  })
_sym_db.RegisterMessage(GetTimeResponse)

SetTimeRequest = _reflection.GeneratedProtocolMessageType('SetTimeRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETTIMEREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.testing.time_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.testing.SetTimeRequest)
  })
_sym_db.RegisterMessage(SetTimeRequest)


DESCRIPTOR._options = None

_TIMESERVICE = _descriptor.ServiceDescriptor(
  name='TimeService',
  full_name='com.daml.ledger.api.v1.testing.TimeService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=388,
  serialized_end=594,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetTime',
    full_name='com.daml.ledger.api.v1.testing.TimeService.GetTime',
    index=0,
    containing_service=None,
    input_type=_GETTIMEREQUEST,
    output_type=_GETTIMERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetTime',
    full_name='com.daml.ledger.api.v1.testing.TimeService.SetTime',
    index=1,
    containing_service=None,
    input_type=_SETTIMEREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_TIMESERVICE)

DESCRIPTOR.services_by_name['TimeService'] = _TIMESERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/admin/config_management_service.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/daml/ledger/api/v1/admin/config_management_service.proto',
  package='com.daml.ledger.api.v1.admin',
  syntax='proto3',
  serialized_options=b'\n\034com.daml.ledger.api.v1.adminB!ConfigManagementServiceOuterClass\252\002\034Com.Daml.Ledger.Api.V1.Admin',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n<com/daml/ledger/api/v1/admin/config_management_service.proto\x12\x1c\x63om.daml.ledger.api.v1.admin\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x15\n\x13GetTimeModelRequest\"u\n\x14GetTimeModelResponse\x12 \n\x18\x63onfiguration_generation\x18\x01 \x01(\x03\x12;\n\ntime_model\x18\x02 \x01(\x0b\x32\'.com.daml.ledger.api.v1.admin.TimeModel\"\xc8\x01\n\x13SetTimeModelRequest\x12\x15\n\rsubmission_id\x18\x01 \x01(\t\x12\x37\n\x13maximum_record_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12 \n\x18\x63onfiguration_generation\x18\x03 \x01(\x03\x12?\n\x0enew_time_model\x18\x04 \x01(\x0b\x32\'.com.daml.ledger.api.v1.admin.TimeModel\"8\n\x14SetTimeModelResponse\x12 \n\x18\x63onfiguration_generation\x18\x01 \x01(\x03\"\xb3\x01\n\tTimeModel\x12:\n\x17\x61vg_transaction_latency\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12+\n\x08min_skew\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12+\n\x08max_skew\x18\x06 \x01(\x0b\x32\x19.google.protobuf.DurationJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04\x32\x87\x02\n\x17\x43onfigManagementService\x12u\n\x0cGetTimeModel\x12\x31.com.daml.ledger.api.v1.admin.GetTimeModelRequest\x1a\x32.com.daml.ledger.api.v1.admin.GetTimeModelResponse\x12u\n\x0cSetTimeModel\x12\x31.com.daml.ledger.api.v1.admin.SetTimeModelRequest\x1a\x32.com.daml.ledger.api.v1.admin.SetTimeModelResponseB`\n\x1c\x63om.daml.ledger.api.v1.adminB!ConfigManagementServiceOuterClass\xaa\x02\x1c\x43om.Daml.Ledger.Api.V1.Adminb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_GETTIMEMODELREQUEST = _descriptor.Descriptor(
  name='GetTimeModelRequest',
  full_name='com.daml.ledger.api.v1.admin.GetTimeModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=159,
  serialized_end=180,
)


_GETTIMEMODELRESPONSE = _descriptor.Descriptor(
  name='GetTimeModelResponse',
  full_name='com.daml.ledger.api.v1.admin.GetTimeModelResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='configuration_generation', full_name='com.daml.ledger.api.v1.admin.GetTimeModelResponse.configuration_generation', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_model', full_name='com.daml.ledger.api.v1.admin.GetTimeModelResponse.time_model', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=182,
  serialized_end=299,
)


_SETTIMEMODELREQUEST = _descriptor.Descriptor(
  name='SetTimeModelRequest',
  full_name='com.daml.ledger.api.v1.admin.SetTimeModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='submission_id', full_name='com.daml.ledger.api.v1.admin.SetTimeModelRequest.submission_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='maximum_record_time', full_name='com.daml.ledger.api.v1.admin.SetTimeModelRequest.maximum_record_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='configuration_generation', full_name='com.daml.ledger.api.v1.admin.SetTimeModelRequest.configuration_generation', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='new_time_model', full_name='com.daml.ledger.api.v1.admin.SetTimeModelRequest.new_time_model', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=302,
  serialized_end=502,
)


_SETTIMEMODELRESPONSE = _descriptor.Descriptor(
  name='SetTimeModelResponse',
  full_name='com.daml.ledger.api.v1.admin.SetTimeModelResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='configuration_generation', full_name='com.daml.ledger.api.v1.admin.SetTimeModelResponse.configuration_generation', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=504,
  serialized_end=560,
)


_TIMEMODEL = _descriptor.Descriptor(
  name='TimeModel',
  full_name='com.daml.ledger.api.v1.admin.TimeModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='avg_transaction_latency', full_name='com.daml.ledger.api.v1.admin.TimeModel.avg_transaction_latency', index=0,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_skew', full_name='com.daml.ledger.api.v1.admin.TimeModel.min_skew', index=1,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_skew', full_name='com.daml.ledger.api.v1.admin.TimeModel.max_skew', index=2,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=563,
  serialized_end=742,
)

_GETTIMEMODELRESPONSE.fields_by_name['time_model'].message_type = _TIMEMODEL
_SETTIMEMODELREQUEST.fields_by_name['maximum_record_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SETTIMEMODELREQUEST.fields_by_name['new_time_model'].message_type = _TIMEMODEL
_TIMEMODEL.fields_by_name['avg_transaction_latency'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_TIMEMODEL.fields_by_name['min_skew'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_TIMEMODEL.fields_by_name['max_skew'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name['GetTimeModelRequest'] = _GETTIMEMODELREQUEST
DESCRIPTOR.message_types_by_name['GetTimeModelResponse'] = _GETTIMEMODELRESPONSE
DESCRIPTOR.message_types_by_name['SetTimeModelRequest'] = _SETTIMEMODELREQUEST
DESCRIPTOR.message_types_by_name['SetTimeModelResponse'] = _SETTIMEMODELRESPONSE
DESCRIPTOR.message_types_by_name['TimeModel'] = _TIMEMODEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetTimeModelRequest = _reflection.GeneratedProtocolMessageType('GetTimeModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMEMODELREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.config_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetTimeModelRequest)
  })
_sym_db.RegisterMessage(GetTimeModelRequest)

GetTimeModelResponse = _reflection.GeneratedProtocolMessageType('GetTimeModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTIMEMODELRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.config_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetTimeModelResponse)
  })
_sym_db.RegisterMessage(GetTimeModelResponse)

SetTimeModelRequest = _reflection.GeneratedProtocolMessageType('SetTimeModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETTIMEMODELREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.config_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.SetTimeModelRequest)
  })
_sym_db.RegisterMessage(SetTimeModelRequest)

SetTimeModelResponse = _reflection.GeneratedProtocolMessageType('SetTimeModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETTIMEMODELRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.config_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.SetTimeModelResponse)
  })
_sym_db.RegisterMessage(SetTimeModelResponse)

TimeModel = _reflection.GeneratedProtocolMessageType('TimeModel', (_message.Message,), {
  'DESCRIPTOR' : _TIMEMODEL,
  '__module__' : 'com.daml.ledger.api.v1.admin.config_management_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.TimeModel)
  })
_sym_db.RegisterMessage(TimeModel)


DESCRIPTOR._options = None

_CONFIGMANAGEMENTSERVICE = _descriptor.ServiceDescriptor(
  name='ConfigManagementService',
  full_name='com.daml.ledger.api.v1.admin.ConfigManagementService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=745,
  serialized_end=1008,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetTimeModel',
    full_name='com.daml.ledger.api.v1.admin.ConfigManagementService.GetTimeModel',
    index=0,
    containing_service=None,
    input_type=_GETTIMEMODELREQUEST,
    output_type=_GETTIMEMODELRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetTimeModel',
    full_name='com.daml.ledger.api.v1.admin.ConfigManagementService.SetTimeModel',
    index=1,
    containing_service=None,
    input_type=_SETTIMEMODELREQUEST,
    output_type=_SETTIMEMODELRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_CONFIGMANAGEMENTSERVICE)

DESCRIPTOR.services_by_name['ConfigManagementService'] = _CONFIGMANAGEMENTSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import ledger_configuration_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2


class LedgerConfigurationServiceStub(object):
    """LedgerConfigurationService allows clients to subscribe to changes of the ledger configuration.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetLedgerConfiguration = channel.unary_stream(
                '/com.daml.ledger.api.v1.LedgerConfigurationService/GetLedgerConfiguration',
                request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationRequest.SerializeToString,
                response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationResponse.FromString,
                )


class LedgerConfigurationServiceServicer(object):
    """LedgerConfigurationService allows clients to subscribe to changes of the ledger configuration.
    """

    def GetLedgerConfiguration(self, request, context):
        """Returns the latest configuration as the first response, and publishes configuration updates in the same stream.
        Errors:
        - ``UNAUTHENTICATED``: if the request does not include a valid access token
        - ``PERMISSION_DENIED``: if the claims in the token are insufficient to perform a given operation
        - ``NOT_FOUND``: if the request does not include a valid ledger id
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LedgerConfigurationServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetLedgerConfiguration': grpc.unary_stream_rpc_method_handler(
                    servicer.GetLedgerConfiguration,
                    request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationRequest.FromString,
                    response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'com.daml.ledger.api.v1.LedgerConfigurationService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class LedgerConfigurationService(object):
    """LedgerConfigurationService allows clients to subscribe to changes of the ledger configuration.
    """

    @staticmethod
    def GetLedgerConfiguration(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/com.daml.ledger.api.v1.LedgerConfigurationService/GetLedgerConfiguration',
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__configuration__service__pb2.GetLedgerConfigurationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module contains configuration parameters used by the rest of the library.
"""

import argparse
from dataclasses import asdict, dataclass, field, fields
from typing import TYPE_CHECKING, Any, Collection, List, Mapping, Optional, Sequence, Type, TypeVar
import warnings

from .. import LOG
from ..model.core import ConfigurationError, Party
from ..util.config_meta import (
    BOOLEAN_TYPE,
    COUNT_TYPE,
    LOG_LEVEL_TYPE,
    PACKAGE_IDS_TYPE,
    PARTIES_TYPE,
    PATH_TYPE,
    PORT_TYPE,
    SECONDS_TYPE,
    STRING_TYPE,
    URL_TYPE,
    VERIFY_SSL_TYPE,
    add_argument,
    config_field,
    config_fields,
)

# If this environment variable is set, is used in place of a configuration file if none is supplied
# on the command-line.
DAZL_CONFIG_ENV = "DAZL_CONFIG"


# Define a type alias for things that quack like an ArgumentParser
if TYPE_CHECKING:
    # noinspection PyProtectedMember
    ArgumentParser = argparse._ActionsContainer


DEFAULT_CONNECT_TIMEOUT_SECONDS = 120


@dataclass(frozen=True)
class URLConfig:
    """
    Configuration for something that requires a URL.
    """

    url: str = config_field(
        "URL where the Ledger API is hosted",
        param_type=URL_TYPE,
        short_alias="u",
        environment_variable="DAML_LEDGER_URL",
        deprecated_alias="participant_url",
    )

    ca_file: Optional[str] = config_field(
        "server certificate authority file", param_type=PATH_TYPE, default_value=None
    )

    cert_file: Optional[str] = config_field("client certificate file", param_type=PATH_TYPE)

    cert_key_file: Optional[str] = config_field(
        "client certificate and key file", param_type=PATH_TYPE
    )

    verify_ssl: Optional[str] = config_field(
        "level of verification to use for SSL", param_type=VERIFY_SSL_TYPE
    )

    poll_interval: Optional[float] = config_field(
        "polling interval for receiving new events", param_type=SECONDS_TYPE, default_value=0.1
    )

    connect_timeout: Optional[float] = config_field(
        "number of seconds before giving up on a connection",
        param_type=SECONDS_TYPE,
        default_value=DEFAULT_CONNECT_TIMEOUT_SECONDS,
    )

    eager_package_fetch: Optional[bool] = config_field(
        "whether to fetch all packages on startup", param_type=BOOLEAN_TYPE, default_value=True
    )

    enable_http_proxy: Optional[bool] = config_field(
        "whether to allow the use of HTTP proxies", param_type=BOOLEAN_TYPE, default_value=True
    )

    package_lookup_timeout: Optional[float] = config_field(
        "number of seconds before giving up on a package",
        param_type=SECONDS_TYPE,
        default_value=DEFAULT_CONNECT_TIMEOUT_SECONDS,
    )

    application_name: Optional[str] = config_field(
        "The name that this application uses to identify itself to the ledger.",
        param_type=STRING_TYPE,
        default_value="DAZL-Client",
    )

    log_level: Optional[int] = config_field(
        "logging level for events for this party",
        param_type=LOG_LEVEL_TYPE,
        environment_variable="DAZL_LOG_LEVEL",
        short_alias="l",
    )


@dataclass(frozen=True)
class _PartyConfig(URLConfig):
    """
    Configuration for a specific party's client.
    """

    max_event_block_size: Optional[int] = config_field(
        "Maximum number of blocks to read in a single call.",
        param_type=COUNT_TYPE,
        default_value=100,
    )

    max_command_batch_size: Optional[int] = config_field(
        "Maximum number of commands to batch up in a single transaction",
        param_type=COUNT_TYPE,
        default_value=100,
    )

    max_command_batch_timeout: Optional[float] = config_field(
        "Maximum number of seconds to wait before sending out a command",
        param_type=SECONDS_TYPE,
        default_value=0.25,
    )

    party_groups: Optional[Collection[str]] = config_field(
        "comma-separated list of broadcast parties", param_type=PARTIES_TYPE
    )


@dataclass(frozen=True)
class _NetworkConfig(URLConfig):
    """
    Configuration for the entire client library and all connected parties.

    The URL for the network is used for calls that do _not_ require a Party (such as retrieving
    basic ledger information), and is also used as a default for party configs that do not supply
    their own URL.
    """

    log_level: Optional[int] = config_field(
        "logging level for events for this party",
        param_type=LOG_LEVEL_TYPE,
        environment_variable="DAZL_LOG_LEVEL",
        short_alias="l",
    )

    max_connection_count: Optional[int] = config_field(
        "Number of concurrent HTTP connections to have outstanding.",
        param_type=COUNT_TYPE,
        default_value=20,
    )

    quiet_timeout: Optional[float] = config_field(
        'Number of seconds to wait after the client "thinks" it\'s done to hang around for',
        param_type=SECONDS_TYPE,
        default_value=1,
    )

    use_acs_service: bool = config_field(
        "Use Active Contract Set service instead of reading from the Transaction event stream",
        param_type=BOOLEAN_TYPE,
        default_value=True,
    )

    idle_timeout: Optional[float] = config_field(
        "Maximum number of seconds of idle activity before automatically closing the client",
        param_type=SECONDS_TYPE,
        default_value=120,
    )

    max_consequence_depth: Optional[int] = config_field(
        "The maximum number of times to wait for all parties to arrive at the same offset"
        + "before failing with an error",
        param_type=COUNT_TYPE,
        default_value=50,
    )

    party_groups: Optional[Collection[str]] = config_field(
        "comma-separated list of broadcast parties", param_type=PARTIES_TYPE
    )

    package_ids: Optional[Collection[str]] = config_field(
        "comma-separated list of package IDs to listen on", param_type=PACKAGE_IDS_TYPE
    )

    server_host: Optional[str] = config_field(
        "Server listening host. Used for OAuth web application flow callbacks.",
        param_type=STRING_TYPE,
        environment_variable="DAZL_SERVER_HOST",
    )

    server_port: Optional[int] = config_field(
        "Server listening port. Used for OAuth web application flow callbacks.",
        param_type=PORT_TYPE,
        environment_variable="DAZL_SERVER_PORT",
    )

    oauth_client_id: Optional[str] = config_field("OAuth client ID", param_type=STRING_TYPE)

    oauth_client_secret: Optional[str] = config_field(
        "OAuth client secret (implies web application flow or backend application flow)",
        param_type=STRING_TYPE,
    )

    oauth_token: Optional[str] = config_field("OAuth token", param_type=STRING_TYPE)

    oauth_refresh_token: Optional[str] = config_field("OAuth refresh token", param_type=STRING_TYPE)

    oauth_id_token: Optional[str] = config_field(
        "OpenID token (JWT formatted)", param_type=STRING_TYPE
    )

    oauth_token_uri: Optional[str] = config_field("OAuth token URL", param_type=STRING_TYPE)

    oauth_redirect_uri: Optional[str] = config_field(
        "OAuth redirect URI (implies web application flow)", param_type=STRING_TYPE
    )

    oauth_auth_url: Optional[str] = config_field(
        "OAuth auth URL (implies mobile application flow)", param_type=STRING_TYPE
    )

    oauth_ca_file: Optional[str] = config_field("OAuth CA CA Bundle File", param_type=STRING_TYPE)

    oauth_audience: Optional[str] = config_field(
        "OAuth audience (Implies Client Credentials Flow)", param_type=STRING_TYPE
    )

    oauth_legacy_username: Optional[str] = config_field(
        "OAuth username (implies legacy application flow)", param_type=STRING_TYPE
    )

    oauth_legacy_password: Optional[str] = config_field(
        "OAuth password (implies legacy application flow)", param_type=STRING_TYPE
    )


@dataclass(frozen=True)
class _FlatConfig(_NetworkConfig, _PartyConfig):
    """
    A configuration object that assumes a single flat key-value structure suitable for parsing
    command-line or environment variable-based values.

    This configuration object is used as an intermediate holding object when parsing parameters
    from environment variables or command-line arguments.
    """

    parties: Collection[Party] = config_field(
        "comma-separated list of parties serviced by a participant node",
        param_type=PARTIES_TYPE,
        long_alias="party",
        short_alias="p",
        environment_variable="DAML_LEDGER_PARTY",
        default_value=(),
    )


@dataclass(frozen=True)
class PartyConfig(_PartyConfig, _NetworkConfig):
    party: Optional[Party] = None


C = TypeVar("C", bound="_TopLevelConfig")


@dataclass(frozen=True)
class _TopLevelConfig:
    @classmethod
    def parse_kwargs(cls: Type[C], *config: "C", **kwargs) -> "C":
        """
        Create a :class:`NetworkConfig` for the configuration settings.

        :param config:
            Instances of :class:`NetworkConfig`` objects to merge together.
        :param kwargs:
            Configuration options that are accepted either at the global level or at the party level.
        :return:
            An instance of :class:`NetworkConfig`.
        """
        configurations = list(config)  # type: List[C]
        if kwargs:
            args_dict = dict(_get_env_defaults())
            args_dict.update(_parse_args_dict(kwargs))
            flat_config = _FlatConfig(**args_dict)
            configurations.append(cls.unflatten(flat_config))

        # TODO: Support merging of more than one configuration object
        return configurations[0].validate()

    @classmethod
    def get_config(cls: Type[C], args: "argparse.Namespace") -> "C":
        """
        Convert an ``argparse.Namespace`` to a fully-formed and valid :class:`NetworkConfig`.
        """
        args_dict = dict(_get_env_defaults())
        args_dict.update(_parse_args_ns(args))
        flat_config = _FlatConfig(**args_dict)
        network_config = cls.unflatten(flat_config)
        return network_config.validate()

    @classmethod
    def unflatten(cls: Type[C], config: "_FlatConfig") -> "C":
        raise NotImplementedError

    def validate(self: C) -> "C":
        """
        Return this config, or raise a ConfigurationError if there is something wrong with the
        parameters of this object.

        This should only be called on fully-formed config objects once all possible values are
        merged together.
        """
        raise NotImplementedError


@dataclass(frozen=True)
class NetworkConfig(_NetworkConfig, _TopLevelConfig):
    parties: "Sequence[PartyConfig]" = field(default_factory=tuple)

    @classmethod
    def unflatten(cls, config: "_FlatConfig") -> "NetworkConfig":
        """
        Convert a :class:`_FlatConfig` to a :class:`NetworkConfig`.
        """
        config_dict = {k: v for k, v in asdict(config).items() if v is not None}
        network_dict = {fld.name: config_dict.get(fld.name) for fld in fields(_NetworkConfig)}
        party_dict = {fld.name: config_dict.get(fld.name) for fld in fields(_PartyConfig)}
        parties = tuple(PartyConfig(party=party, **party_dict) for party in config.parties)
        return NetworkConfig(parties=parties, **network_dict)

    def validate(self) -> "NetworkConfig":
        failures = set()
        LOG.debug("Configuration: %s", self)
        if self.parties:
            for party_config in self.parties:
                if not party_config.url:
                    failures.add(f"Party {party_config.party} has no Ledger API URL")

        if failures:
            raise ConfigurationError(failures)

        return self


@dataclass(frozen=True)
class AnonymousNetworkConfig(_NetworkConfig, _TopLevelConfig):
    """
    A configuration object that contains all of the necessary parameters to configure a network
    with no parties.
    """

    @classmethod
    def unflatten(cls, config: "_FlatConfig") -> "AnonymousNetworkConfig":
        """
        Convert a :class:`_FlatConfig` to a :class:`NetworkConfig`.
        """
        config_dict = {k: v for k, v in asdict(config).items() if v is not None}
        network_dict = {
            fld.name: config_dict.get(fld.name) for fld in fields(AnonymousNetworkConfig)
        }
        return AnonymousNetworkConfig(**network_dict)

    def validate(self) -> "AnonymousNetworkConfig":
        failures = []

        if not self.url:
            failures.append("URL must be specified")

        if failures:
            raise ConfigurationError(failures)

        return self


def configure_parser(
    arg_parser: "ArgumentParser", config_file_support: bool = False, parties: bool = True
):
    """
    Add standard options to an arg parser (later to be extracted out by ``get_config``).

    :param arg_parser:
        The ArgumentParser to add arguments to.
    :param config_file_support:
        ``True`` to enable reading configuration from a config file.
    :param parties:
        ``True`` to mandate (and validate) party-specific settings; ``False`` to omit them.
    """
    if config_file_support:
        from os import environ

        default_config = environ.get(DAZL_CONFIG_ENV)
        group = arg_parser.add_argument_group("Configuration Settings")
        group.add_argument("--config", help="path to a YAML config file", default=default_config)

    if parties:
        party_config_fields = [fld.name for fld, _ in config_fields(_PartyConfig)]

        group = arg_parser.add_argument_group("Overall Settings")
        for fld, param in config_fields(_FlatConfig):
            if fld.name not in party_config_fields:
                add_argument(group, fld.name, param)

        group = arg_parser.add_argument_group("Per-Party Settings")
        for fld, param in config_fields(_PartyConfig):
            add_argument(group, fld.name, param)

    else:
        for fld, param in config_fields(URLConfig):
            add_argument(arg_parser, fld.name, param)

    return arg_parser


def fetch_config(path_or_url: "Optional[str]") -> "Optional[str]":
    """
    Attempts to fetch a config file from what looks like either a path or a URL.

    :param path_or_url: A file path or a URL.
    :return: The body of the config file.
    """
    if not path_or_url:
        return None

    from urllib.error import HTTPError, URLError
    from urllib.parse import urlparse
    from urllib.request import urlopen

    u = urlparse(path_or_url)
    if not u.scheme or u.scheme == "file":
        try:
            with open(u.path, "r") as buf:
                config_str = buf.read()
        except FileNotFoundError:
            raise ConfigurationError([f"Config file not found: {u.path}"])
    else:
        try:
            with urlopen(path_or_url) as buf:
                config_str = buf.read().decode("utf8")
        except HTTPError as error:
            raise ConfigurationError([f"HTTP {error.code}: {path_or_url}"])
        except URLError as error:
            if isinstance(error.args[0], ConnectionRefusedError):
                raise ConfigurationError([f"HTTP unreachable: {path_or_url}"])
            else:
                raise ConfigurationError([error.args[0]])

    return config_str


def _get_env_defaults() -> "Mapping[str, Any]":
    """
    Produce a mapping of configuration keys to values set in the environment through environment
    variables.
    """
    from os import getenv

    kwargs = {}
    for fld, param in config_fields(_FlatConfig):
        if param.environment_variable:
            value = getenv(param.environment_variable)
            if value:
                kwargs[fld.name] = param.param_type.value_ctor(value)
    return kwargs


def _parse_args_ns(args: "argparse.Namespace") -> "Mapping[str, Any]":
    """
    Convert the values
    :param args:
    :return:
    """
    return {
        fld.name: getattr(args, fld.name) for fld in fields(_FlatConfig) if hasattr(args, fld.name)
    }


def _parse_args_dict(d: "Mapping[str, Any]") -> "Mapping[str, Any]":
    """
    Convert the key-value pairs in this mapping to keys that are defined on :class:`_FlatConfig`.

    :param d:
    :return:
    """
    kwargs = {}
    unknown_fields = set()
    for fld, param in config_fields(_FlatConfig):
        value = d.get(fld.name)
        if value is None:
            for key in param.long_aliases:
                if d.get(key) is not None:
                    value = d.get(key)
                    break

        if value is None:
            for key in param.deprecated_aliases:
                if d.get(key) is not None:
                    warnings.warn(
                        f"The {key} option is deprecated. Please use {fld.name} instead.",
                        DeprecationWarning,
                        stacklevel=2,
                    )
                    value = d.get(key)
                    break

        if value is not None:
            if param.param_type.value_ctor is not None:
                try:
                    value = param.param_type.value_ctor(value)
                except:
                    LOG.exception("Config key: %r; could not parse value: %r", fld.name, value)

            kwargs[fld.name] = value

    if unknown_fields:
        raise ValueError(f"unknown kwargs: {sorted(unknown_fields)}")
    return kwargs



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/admin/participant_pruning_service.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/daml/ledger/api/v1/admin/participant_pruning_service.proto',
  package='com.daml.ledger.api.v1.admin',
  syntax='proto3',
  serialized_options=b'\n\034com.daml.ledger.api.v1.adminB#ParticipantPruningServiceOuterClass\252\002\034Com.Daml.Ledger.Api.V1.Admin',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n>com/daml/ledger/api/v1/admin/participant_pruning_service.proto\x12\x1c\x63om.daml.ledger.api.v1.admin\":\n\x0cPruneRequest\x12\x13\n\x0bprune_up_to\x18\x01 \x01(\t\x12\x15\n\rsubmission_id\x18\x02 \x01(\t\"\x0f\n\rPruneResponse2}\n\x19ParticipantPruningService\x12`\n\x05Prune\x12*.com.daml.ledger.api.v1.admin.PruneRequest\x1a+.com.daml.ledger.api.v1.admin.PruneResponseBb\n\x1c\x63om.daml.ledger.api.v1.adminB#ParticipantPruningServiceOuterClass\xaa\x02\x1c\x43om.Daml.Ledger.Api.V1.Adminb\x06proto3'
)




_PRUNEREQUEST = _descriptor.Descriptor(
  name='PruneRequest',
  full_name='com.daml.ledger.api.v1.admin.PruneRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='prune_up_to', full_name='com.daml.ledger.api.v1.admin.PruneRequest.prune_up_to', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='submission_id', full_name='com.daml.ledger.api.v1.admin.PruneRequest.submission_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=96,
  serialized_end=154,
)


_PRUNERESPONSE = _descriptor.Descriptor(
  name='PruneResponse',
  full_name='com.daml.ledger.api.v1.admin.PruneResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=156,
  serialized_end=171,
)

DESCRIPTOR.message_types_by_name['PruneRequest'] = _PRUNEREQUEST
DESCRIPTOR.message_types_by_name['PruneResponse'] = _PRUNERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PruneRequest = _reflection.GeneratedProtocolMessageType('PruneRequest', (_message.Message,), {
  'DESCRIPTOR' : _PRUNEREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.admin.participant_pruning_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.PruneRequest)
  })
_sym_db.RegisterMessage(PruneRequest)

PruneResponse = _reflection.GeneratedProtocolMessageType('PruneResponse', (_message.Message,), {
  'DESCRIPTOR' : _PRUNERESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.admin.participant_pruning_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.PruneResponse)
  })
_sym_db.RegisterMessage(PruneResponse)


DESCRIPTOR._options = None

_PARTICIPANTPRUNINGSERVICE = _descriptor.ServiceDescriptor(
  name='ParticipantPruningService',
  full_name='com.daml.ledger.api.v1.admin.ParticipantPruningService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=173,
  serialized_end=298,
  methods=[
  _descriptor.MethodDescriptor(
    name='Prune',
    full_name='com.daml.ledger.api.v1.admin.ParticipantPruningService.Prune',
    index=0,
    containing_service=None,
    input_type=_PRUNEREQUEST,
    output_type=_PRUNERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_PARTICIPANTPRUNINGSERVICE)

DESCRIPTOR.services_by_name['ParticipantPruningService'] = _PARTICIPANTPRUNINGSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/event.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import value_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/daml/ledger/api/v1/event.proto',
  package='com.daml.ledger.api.v1',
  syntax='proto3',
  serialized_options=b'\n\026com.daml.ledger.api.v1B\017EventOuterClass\252\002\026Com.Daml.Ledger.Api.V1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\"com/daml/ledger/api/v1/event.proto\x12\x16\x63om.daml.ledger.api.v1\x1a\"com/daml/ledger/api/v1/value.proto\x1a\x1egoogle/protobuf/wrappers.proto\"\x95\x01\n\x05\x45vent\x12\x37\n\x07\x63reated\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.CreatedEventH\x00\x12\x39\n\x08\x61rchived\x18\x03 \x01(\x0b\x32%.com.daml.ledger.api.v1.ArchivedEventH\x00\x42\x07\n\x05\x65ventJ\x04\x08\x02\x10\x03R\texercised\"\xd4\x02\n\x0c\x43reatedEvent\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x37\n\x0btemplate_id\x18\x03 \x01(\x0b\x32\".com.daml.ledger.api.v1.Identifier\x12\x33\n\x0c\x63ontract_key\x18\x07 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.Value\x12\x38\n\x10\x63reate_arguments\x18\x04 \x01(\x0b\x32\x1e.com.daml.ledger.api.v1.Record\x12\x17\n\x0fwitness_parties\x18\x05 \x03(\t\x12\x13\n\x0bsignatories\x18\x08 \x03(\t\x12\x11\n\tobservers\x18\t \x03(\t\x12\x34\n\x0e\x61greement_text\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.StringValue\"\x88\x01\n\rArchivedEvent\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x37\n\x0btemplate_id\x18\x03 \x01(\x0b\x32\".com.daml.ledger.api.v1.Identifier\x12\x17\n\x0fwitness_parties\x18\x04 \x03(\t\"\xd9\x02\n\x0e\x45xercisedEvent\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\t\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x37\n\x0btemplate_id\x18\x03 \x01(\x0b\x32\".com.daml.ledger.api.v1.Identifier\x12\x0e\n\x06\x63hoice\x18\x05 \x01(\t\x12\x36\n\x0f\x63hoice_argument\x18\x06 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.Value\x12\x16\n\x0e\x61\x63ting_parties\x18\x07 \x03(\t\x12\x11\n\tconsuming\x18\x08 \x01(\x08\x12\x17\n\x0fwitness_parties\x18\n \x03(\t\x12\x17\n\x0f\x63hild_event_ids\x18\x0b \x03(\t\x12\x36\n\x0f\x65xercise_result\x18\x0c \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.ValueJ\x04\x08\x04\x10\x05J\x04\x08\t\x10\nBB\n\x16\x63om.daml.ledger.api.v1B\x0f\x45ventOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3'
  ,
  dependencies=[com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])




_EVENT = _descriptor.Descriptor(
  name='Event',
  full_name='com.daml.ledger.api.v1.Event',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='created', full_name='com.daml.ledger.api.v1.Event.created', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='archived', full_name='com.daml.ledger.api.v1.Event.archived', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='event', full_name='com.daml.ledger.api.v1.Event.event',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=131,
  serialized_end=280,
)


_CREATEDEVENT = _descriptor.Descriptor(
  name='CreatedEvent',
  full_name='com.daml.ledger.api.v1.CreatedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_id', full_name='com.daml.ledger.api.v1.CreatedEvent.event_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contract_id', full_name='com.daml.ledger.api.v1.CreatedEvent.contract_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='template_id', full_name='com.daml.ledger.api.v1.CreatedEvent.template_id', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contract_key', full_name='com.daml.ledger.api.v1.CreatedEvent.contract_key', index=3,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='create_arguments', full_name='com.daml.ledger.api.v1.CreatedEvent.create_arguments', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='witness_parties', full_name='com.daml.ledger.api.v1.CreatedEvent.witness_parties', index=5,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='signatories', full_name='com.daml.ledger.api.v1.CreatedEvent.signatories', index=6,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='observers', full_name='com.daml.ledger.api.v1.CreatedEvent.observers', index=7,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='agreement_text', full_name='com.daml.ledger.api.v1.CreatedEvent.agreement_text', index=8,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=283,
  serialized_end=623,
)


_ARCHIVEDEVENT = _descriptor.Descriptor(
  name='ArchivedEvent',
  full_name='com.daml.ledger.api.v1.ArchivedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_id', full_name='com.daml.ledger.api.v1.ArchivedEvent.event_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contract_id', full_name='com.daml.ledger.api.v1.ArchivedEvent.contract_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='template_id', full_name='com.daml.ledger.api.v1.ArchivedEvent.template_id', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='witness_parties', full_name='com.daml.ledger.api.v1.ArchivedEvent.witness_parties', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=626,
  serialized_end=762,
)


_EXERCISEDEVENT = _descriptor.Descriptor(
  name='ExercisedEvent',
  full_name='com.daml.ledger.api.v1.ExercisedEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_id', full_name='com.daml.ledger.api.v1.ExercisedEvent.event_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contract_id', full_name='com.daml.ledger.api.v1.ExercisedEvent.contract_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='template_id', full_name='com.daml.ledger.api.v1.ExercisedEvent.template_id', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice', full_name='com.daml.ledger.api.v1.ExercisedEvent.choice', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choice_argument', full_name='com.daml.ledger.api.v1.ExercisedEvent.choice_argument', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acting_parties', full_name='com.daml.ledger.api.v1.ExercisedEvent.acting_parties', index=5,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='consuming', full_name='com.daml.ledger.api.v1.ExercisedEvent.consuming', index=6,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='witness_parties', full_name='com.daml.ledger.api.v1.ExercisedEvent.witness_parties', index=7,
      number=10, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='child_event_ids', full_name='com.daml.ledger.api.v1.ExercisedEvent.child_event_ids', index=8,
      number=11, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exercise_result', full_name='com.daml.ledger.api.v1.ExercisedEvent.exercise_result', index=9,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=765,
  serialized_end=1110,
)

_EVENT.fields_by_name['created'].message_type = _CREATEDEVENT
_EVENT.fields_by_name['archived'].message_type = _ARCHIVEDEVENT
_EVENT.oneofs_by_name['event'].fields.append(
  _EVENT.fields_by_name['created'])
_EVENT.fields_by_name['created'].containing_oneof = _EVENT.oneofs_by_name['event']
_EVENT.oneofs_by_name['event'].fields.append(
  _EVENT.fields_by_name['archived'])
_EVENT.fields_by_name['archived'].containing_oneof = _EVENT.oneofs_by_name['event']
_CREATEDEVENT.fields_by_name['template_id'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_CREATEDEVENT.fields_by_name['contract_key'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
_CREATEDEVENT.fields_by_name['create_arguments'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._RECORD
_CREATEDEVENT.fields_by_name['agreement_text'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_ARCHIVEDEVENT.fields_by_name['template_id'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_EXERCISEDEVENT.fields_by_name['template_id'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_EXERCISEDEVENT.fields_by_name['choice_argument'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
_EXERCISEDEVENT.fields_by_name['exercise_result'].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
DESCRIPTOR.message_types_by_name['Event'] = _EVENT
DESCRIPTOR.message_types_by_name['CreatedEvent'] = _CREATEDEVENT
DESCRIPTOR.message_types_by_name['ArchivedEvent'] = _ARCHIVEDEVENT
DESCRIPTOR.message_types_by_name['ExercisedEvent'] = _EXERCISEDEVENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), {
  'DESCRIPTOR' : _EVENT,
  '__module__' : 'com.daml.ledger.api.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.Event)
  })
_sym_db.RegisterMessage(Event)

CreatedEvent = _reflection.GeneratedProtocolMessageType('CreatedEvent', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDEVENT,
  '__module__' : 'com.daml.ledger.api.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.CreatedEvent)
  })
_sym_db.RegisterMessage(CreatedEvent)

ArchivedEvent = _reflection.GeneratedProtocolMessageType('ArchivedEvent', (_message.Message,), {
  'DESCRIPTOR' : _ARCHIVEDEVENT,
  '__module__' : 'com.daml.ledger.api.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ArchivedEvent)
  })
_sym_db.RegisterMessage(ArchivedEvent)

ExercisedEvent = _reflection.GeneratedProtocolMessageType('ExercisedEvent', (_message.Message,), {
  'DESCRIPTOR' : _EXERCISEDEVENT,
  '__module__' : 'com.daml.ledger.api.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ExercisedEvent)
  })
_sym_db.RegisterMessage(ExercisedEvent)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module contains helper utilities for working within the constraints of Python's type system.
"""

from types import MappingProxyType
from typing import Any, Dict, Optional, Type, TypeVar

K = TypeVar("K")
V = TypeVar("V")
T = TypeVar("T")


def safe_cast(typ: Type[T], val: Any) -> T:
    """
    Safely casts an item to the specified type. Unlike ``typing.cast``, this method will actually
    throw an exception if types do not match.
    """
    if isinstance(typ, type):
        if isinstance(val, typ):
            return val
        else:
            raise ValueError(f"expected a value of type {typ} here (got {val} instead)")
    else:
        raise Exception(f"safe_cast can only be used with actual types (got {typ})")


def safe_optional_cast(typ: Type[T], val: Any) -> Optional[T]:
    """
    Safely casts an item to the specified type. Unlike ``typing.cast``, this method will actually
    throw an exception if types do not match.
    """
    if val is None:
        return None

    if isinstance(typ, type):
        if isinstance(val, typ):
            return val
        else:
            raise ValueError(f"expected a value of type {typ} here (got {val} instead)")
    else:
        raise Exception("safe_cast can only be used with actual types")


def safe_dict_cast(key_type: Type[K], value_type: Type[V], val: Any) -> Dict[K, V]:
    if isinstance(val, (dict, MappingProxyType)):
        for key, value in val.items():
            safe_cast(key_type, key)
            safe_cast(value_type, value)
        return val  # type: ignore
    else:
        raise ValueError(f"expected a dict (got {val} instead)")


def unpack_optional(typ: Any) -> Optional[Any]:
    """
    Return the single non-``NoneType`` type for an ``Optional[T]`` expression, or ``None`` if the
    given type does not match that pattern.

    :param typ: The type to attempt to unpack.
    :return: The type parameter, or ``None`` if the specified type is not ``Optional``.

    >>> unpack_optional(Optional[int])
    <class 'int'>

    >>> unpack_optional(Union[str, None])
    <class 'str'>

    >>> unpack_optional(Union[Optional[float]])
    <class 'float'>

    >>> unpack_optional(Union[float, None, int])
    """
    # noinspection PyUnresolvedReferences,PyProtectedMember
    try:
        from typing import Union
    except ImportError:
        from typing import _Union as Union  # type: ignore

    if type(typ) is Union:
        type_args = list(typ.__args__)
        if len(type_args) == 2:
            try:
                type_args.remove(type(None))
                return type_args[0]
            except ValueError:
                return None

    return None



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

import warnings

warnings.warn("dazl.protocols.v0 is deprecated", DeprecationWarning, stacklevel=2)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/version_service.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='com/daml/ledger/api/v1/version_service.proto',
  package='com.daml.ledger.api.v1',
  syntax='proto3',
  serialized_options=b'\n\026com.daml.ledger.api.v1B\030VersionServiceOuterClass\252\002\026Com.Daml.Ledger.Api.V1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n,com/daml/ledger/api/v1/version_service.proto\x12\x16\x63om.daml.ledger.api.v1\"/\n\x1aGetLedgerApiVersionRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\".\n\x1bGetLedgerApiVersionResponse\x12\x0f\n\x07version\x18\x01 \x01(\t2\x90\x01\n\x0eVersionService\x12~\n\x13GetLedgerApiVersion\x12\x32.com.daml.ledger.api.v1.GetLedgerApiVersionRequest\x1a\x33.com.daml.ledger.api.v1.GetLedgerApiVersionResponseBK\n\x16\x63om.daml.ledger.api.v1B\x18VersionServiceOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3'
)




_GETLEDGERAPIVERSIONREQUEST = _descriptor.Descriptor(
  name='GetLedgerApiVersionRequest',
  full_name='com.daml.ledger.api.v1.GetLedgerApiVersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ledger_id', full_name='com.daml.ledger.api.v1.GetLedgerApiVersionRequest.ledger_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=72,
  serialized_end=119,
)


_GETLEDGERAPIVERSIONRESPONSE = _descriptor.Descriptor(
  name='GetLedgerApiVersionResponse',
  full_name='com.daml.ledger.api.v1.GetLedgerApiVersionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='com.daml.ledger.api.v1.GetLedgerApiVersionResponse.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=167,
)

DESCRIPTOR.message_types_by_name['GetLedgerApiVersionRequest'] = _GETLEDGERAPIVERSIONREQUEST
DESCRIPTOR.message_types_by_name['GetLedgerApiVersionResponse'] = _GETLEDGERAPIVERSIONRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetLedgerApiVersionRequest = _reflection.GeneratedProtocolMessageType('GetLedgerApiVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERAPIVERSIONREQUEST,
  '__module__' : 'com.daml.ledger.api.v1.version_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerApiVersionRequest)
  })
_sym_db.RegisterMessage(GetLedgerApiVersionRequest)

GetLedgerApiVersionResponse = _reflection.GeneratedProtocolMessageType('GetLedgerApiVersionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETLEDGERAPIVERSIONRESPONSE,
  '__module__' : 'com.daml.ledger.api.v1.version_service_pb2'
  # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerApiVersionResponse)
  })
_sym_db.RegisterMessage(GetLedgerApiVersionResponse)


DESCRIPTOR._options = None

_VERSIONSERVICE = _descriptor.ServiceDescriptor(
  name='VersionService',
  full_name='com.daml.ledger.api.v1.VersionService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=170,
  serialized_end=314,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetLedgerApiVersion',
    full_name='com.daml.ledger.api.v1.VersionService.GetLedgerApiVersion',
    index=0,
    containing_service=None,
    input_type=_GETLEDGERAPIVERSIONREQUEST,
    output_type=_GETLEDGERAPIVERSIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_VERSIONSERVICE)

DESCRIPTOR.services_by_name['VersionService'] = _VERSIONSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from typing import Any

from ..damlast.daml_lf_1 import TypeConName
from ..model.writing import (
    AbstractSerializer,
    CreateAndExerciseCommand,
    CreateCommand,
    ExerciseByKeyCommand,
    ExerciseCommand,
)
from ..prim import ContractId
from ..values import CanonicalMapper


class ValidateSerializer(AbstractSerializer):
    """
    A serializer that doesn't actually write to an on-wire form, but merely does lightweight
    conversion and give on-wire serialization the best chance of succeeding.

    This class essentially handles type conversion before a command is fully created and sent to
    a background thread for eventual writing to the Ledger API.
    """

    mapper = CanonicalMapper()

    def serialize_create_command(
        self, name: "TypeConName", template_args: "Any"
    ) -> "CreateCommand":
        return CreateCommand(template=name, arguments=template_args)

    def serialize_exercise_command(
        self, contract_id: "ContractId", choice_name: str, choice_args: "Any"
    ) -> "ExerciseCommand":
        return ExerciseCommand(contract=contract_id, choice=choice_name, arguments=choice_args)

    def serialize_exercise_by_key_command(
        self,
        template_name: "TypeConName",
        key_arguments: Any,
        choice_name: str,
        choice_arguments: "Any",
    ) -> "ExerciseByKeyCommand":
        return ExerciseByKeyCommand(
            template=template_name,
            contract_key=key_arguments,
            choice=choice_name,
            choice_argument=choice_arguments,
        )

    def serialize_create_and_exercise_command(
        self,
        template_name: "TypeConName",
        create_arguments: Any,
        choice_name: str,
        choice_arguments: "Any",
    ) -> "CreateAndExerciseCommand":
        return CreateAndExerciseCommand(
            template=template_name,
            arguments=create_arguments,
            choice=choice_name,
            choice_argument=choice_arguments,
        )
